@@ -1392,7 +1392,7 @@ Window::Controller *Application::windowForShowingHistory(
 
 Window::Controller *Application::windowForShowingForum(
 		not_null<Data::Forum*> forum) const {
-	const auto tabs = forum->channel()->useSubsectionTabs();
+	const auto tabs = forum->bot() || forum->peer()->useSubsectionTabs();
 	const auto id = Window::SeparateId(
 		tabs ? Window::SeparateType::Chat : Window::SeparateType::Forum,
 		forum->history());
