@@ -451,7 +451,7 @@ void BoostBox(
 			: tr::lng_boost_channel_button();
 	}) | rpl::flatten_latest();
 
-	const auto button = box->addButton(rpl::duplicate(submit), [=] {
+	box->addButton(rpl::duplicate(submit), [=] {
 		if (state->submitted) {
 			return;
 		} else if (state->data.current().nextLevelBoosts > 0
@@ -519,17 +519,6 @@ void BoostBox(
 			box->closeBox();
 		}
 	});
-
-	rpl::combine(
-		std::move(submit),
-		box->widthValue()
-	) | rpl::start_with_next([=](const QString &, int width) {
-		const auto &padding = st::boostBox.buttonPadding;
-		button->resizeToWidth(width
-			- padding.left()
-			- padding.right());
-		button->moveToLeft(padding.left(), button->y());
-	}, button->lifetime());
 }
 
 object_ptr<Ui::RpWidget> MakeLinkLabel(
@@ -763,14 +752,16 @@ void AskBoostBox(
 			box,
 			std::move(title),
 			st::boostCenteredTitle),
-		st::boxRowPadding + QMargins(0, st::boostTitleSkip, 0, 0));
+		st::boxRowPadding + QMargins(0, st::boostTitleSkip, 0, 0),
+		style::al_top);
 	box->addRow(
 		object_ptr<Ui::FlatLabel>(
 			box,
 			std::move(text),
 			st::boostText),
 		(st::boxRowPadding
-			+ QMargins(0, st::boostTextSkip, 0, st::boostBottomSkip)));
+			+ QMargins(0, st::boostTextSkip, 0, st::boostBottomSkip)),
+		style::al_top);
 
 	auto stats = object_ptr<Ui::IconButton>(box, st::boostLinkStatsButton);
 	stats->setClickedCallback(openStatistics);
@@ -788,20 +779,10 @@ void AskBoostBox(
 		data.group);
 
 	auto submit = tr::lng_boost_channel_ask_button();
-	const auto button = box->addButton(rpl::duplicate(submit), [=] {
+	box->addButton(rpl::duplicate(submit), [=] {
 		QGuiApplication::clipboard()->setText(data.link);
 		box->uiShow()->showToast(tr::lng_username_copied(tr::now));
 	});
-	rpl::combine(
-		std::move(submit),
-		box->widthValue()
-	) | rpl::start_with_next([=](const QString &, int width) {
-		const auto &padding = st::boostBox.buttonPadding;
-		button->resizeToWidth(width
-			- padding.left()
-			- padding.right());
-		button->moveToLeft(padding.left(), button->y());
-	}, button->lifetime());
 }
 
 void FillBoostLimit(
